package com.persagy.dmp.admin.jms;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@Configuration
public class JmsConfig {


	@Value("${persagy.dmp.exchange}")
	private String exchange;

	@Value("${persagy.dmp.admin.routingKey}")
	private String adminRoutingKey;

	@Value("${persagy.dmp.admin.queue}")
	private String adminQueue;

	@Autowired
	private MessageProcesser messageProcesser;

	@Bean
	public TopicExchange exchage() {
		return new TopicExchange(exchange);
	}

	@Bean
	public Queue queue() {
		return new Queue(adminQueue, true);
	}

	// admin -> admin
	@Bean
	public Binding rwdBinding() {
		return BindingBuilder.bind(queue()).to(exchage()).with(adminRoutingKey);
	}

	@RabbitListener(queues = "${persagy.dmp.admin.queue}")    //监听器监听指定的Queue
	public void processC(DmpMessage message) {
		log.info("============================== Receive {}: {}", adminQueue, JacksonMapper.toSimpleJson(message));
		messageProcesser.listen(message);
	}

}
